@@ -4,6 +4,47 @@
 
 ###############################
 # Definition with initialise
+
+#' Class "SaemixRes"
+#' 
+#' An object of the SaemixRes class, representing the results of a fit through the SAEM algorithm.
+#' 
+#' @name SaemixRes-class 
+#' @docType class
+#' @aliases SaemixRes SaemixRes-class
+#' @aliases [<-,SaemixRes-method plot,SaemixRes print,SaemixRes showall,SaemixRes show,SaemixRes 
+#' 
+#' @section Objects from the Class: 
+#' An object of the SaemixRes class is returned as part of a SaemixObject object after a call to \code{\link{saemix}} and contain the following slots:
+#'   \describe{
+#'     \item{\code{name.fixed}:}{a vector containing the names of the fixed parameters in the model}
+#'     }
+#' @section Methods:
+#'   \describe{
+#'     \item{[<-}{\code{signature(x = "SaemixRes")}: replace elements of object}
+#'     \item{[}{\code{signature(x = "SaemixRes")}: access elements of object}
+#'     \item{initialize}{\code{signature(.Object = "SaemixRes")}: internal function to initialise object, not to be used}
+#'     \item{print}{\code{signature(x = "SaemixRes")}: prints details about the object (more extensive than show)}
+#'     \item{read}{\code{signature(object = "SaemixRes")}: internal function, not to be used }
+#'     \item{showall}{\code{signature(object = "SaemixRes")}: shows all the elements in the object}
+#'     \item{show}{\code{signature(object = "SaemixRes")}: prints details about the object}
+#'     \item{summary}{\code{signature(object = "SaemixRes")}: summary of the results. Returns a list with a number of elements extracted from the results ().}
+#' 	 }
+#' @references Kuhn E, Lavielle M. Maximum likelihood estimation in nonlinear mixed effects models. Computational Statistics and Data Analysis 49, 4 (2005), 1020-1038.
+#' 
+#' Comets E, Lavenu A, Lavielle M. SAEMIX, an R version of the SAEM algorithm. 20th meeting of the Population Approach Group in Europe, Athens, Greece (2011), Abstr 2173.
+#' @author Emmanuelle Comets \email{emmanuelle.comets@@inserm.fr}
+#' @author Audrey Lavenu
+#' @author Marc Lavielle.
+#' @seealso \code{\link{saemixData}} \code{\link{SaemixModel}} \code{\link{saemixControl}} \code{\link{saemix}}
+#' @examples
+#' methods(class="SaemixRes")
+#' 
+#' showClass("SaemixRes")
+#' 
+#' @keywords classes
+#' @exportClass SaemixRes
+
 setClass(
   Class="SaemixRes",
   representation=representation(
@@ -29,18 +70,19 @@ setClass(
     indx.res="numeric",	# index of param of residual errors estimated (was indx.res)
     MCOV="matrix",		# 
 # Individual parameters
-    cond.mean.phi="matrix",	# Cond mean estimates of Phi
-    cond.mean.psi="matrix",	# Cond mean estimates of Psi
-    cond.var.phi="matrix",
-    cond.mean.eta="matrix",	# Cond mean estimates of Eta
+    cond.mean.phi="matrix",	# Cond mean estimates of Phi (mean of conditional distribution)
+    cond.mean.psi="matrix",	# Cond mean estimates of Psi (h(cond.mean.phi))
+    cond.var.phi="matrix",  # Variability on cond.mean.phi (var of conditional distribution)
+    cond.mean.eta="matrix",	# Cond mean estimates of eta (CHECK FORMULA)
     cond.shrinkage="numeric",	# Shrinkage for cond mean estimates of Eta
-    mean.phi="matrix",
-    map.psi="data.frame",	# MAP estimates of individual parameters
+    mean.phi="matrix",  # population estimate (Ci*mu) including covariate effects
+    map.psi="data.frame",	# MAP estimates of individual parameters 
     map.phi="data.frame",	# MAP estimates of phi
-    map.eta="matrix",		# ETAs corresponding to the MAP estimates
+    map.eta="matrix",		# ETAs corresponding to the MAP estimates (computed as map.phi-COV%*%MCOV in compute.eta.map)
     map.shrinkage="numeric", 	# shrinkage on MAP estimates
     phi="matrix",
-    phi.samp="array",		# nb.chains samples in the individual conditional distributions
+    psi.samp="array",		# nb.chains samples in the individual conditional distributions (psi)
+    phi.samp="array",		# nb.chains samples in the individual conditional distributions (phi)
     phi.samp.var="array",	# variance of samples
 # Statistical criteria
     ll.lin="numeric",		# for each method (linearisation, IS, GQ)
@@ -59,6 +101,7 @@ setClass(
     ppred="numeric",		# vector of population predictions with MAP
     ipred="numeric",		# vector of individual predictions with MAP
     icpred="numeric",		# vector of individual predictions with conditional estimates
+    ires="numeric",		  # vector of individual residuals with MAP (ipred-x)
     iwres="numeric",		# vector of individual weighted residuals with MAP
     icwres="numeric",		# vector of individual weighted residuals with conditional estimates
     wres="numeric",		# vector of WRES (population weighted residuals)
@@ -74,6 +117,9 @@ setClass(
 ###############################
 # initialize
 
+#' @rdname initialize-methods
+#' @exportMethod initialize
+
 setMethod(
   f="initialize",
   signature="SaemixRes",
@@ -126,6 +172,19 @@ setMethod(
 ####			SaemixRes class - accesseur				####
 ####################################################################################
 
+##' Get/set methods for SaemixRes object
+##' 
+##' Access slots of a SaemixRes object using the object["slot"] format
+##' 
+#' @param x object
+#' @param i element to be replaced
+#' @param j element to replace with
+#' @param drop whether to drop unused dimensions
+#' @keywords methods
+#' @exportMethod [
+#' @exportMethod [<-
+
+
 # Getteur
 setMethod(
   f ="[",
@@ -165,6 +224,7 @@ setMethod(
     "map.shrinkage"={return(x@map.shrinkage)},
     "phi"={return(x@phi)},
     "phi.samp"={return(x@phi.samp)},
+    "psi.samp"={return(x@psi.samp)},
     "phi.samp.var"={return(x@phi.samp.var)},
     "ll.lin"={return(x@ll.lin)},
     "aic.lin"={return(x@aic.lin)},
@@ -181,6 +241,7 @@ setMethod(
     "ppred"={return(x@ppred)},
     "ipred"={return(x@ipred)},
     "icpred"={return(x@icpred)},
+    "ires"={return(x@ires)},
     "iwres"={return(x@iwres)},
     "icwres"={return(x@icwres)},
     "wres"={return(x@wres)},
@@ -231,6 +292,7 @@ setReplaceMethod(
     "map.shrinkage"={x@map.shrinkage<-value},
     "phi"={x@phi<-value},
     "phi.samp"={x@phi.samp<-value},
+    "psi.samp"={x@psi.samp<-value},
     "phi.samp.var"={x@phi.samp.var<-value},
     "ll.lin"={x@ll.lin<-value},
     "aic.lin"={x@aic.lin<-value},
@@ -247,6 +309,7 @@ setReplaceMethod(
     "ppred"={x@ppred<-value},
     "ipred"={x@ipred<-value},
     "icpred"={x@icpred<-value},
+    "ires"={x@ires<-value},
     "iwres"={x@iwres<-value},
     "icwres"={x@icwres<-value},
     "wres"={x@wres<-value},
@@ -263,6 +326,8 @@ setReplaceMethod(
 ####			SaemixRes class - method to print/show data		####
 ####################################################################################
 
+#' @exportMethod print
+
 setMethod("print","SaemixRes",
   function(x,digits=2,map=FALSE,...) {
 #    cat("Nonlinear mixed-effects model fit by the SAEM algorithm\n")
@@ -363,6 +428,8 @@ setMethod("print","SaemixRes",
   }
 )
 
+#' @exportMethod show
+
 setMethod("show","SaemixRes",
   function(object) {
 #    cat("Nonlinear mixed-effects model fit by the SAEM algorithm\n")
@@ -441,6 +508,10 @@ setMethod("show","SaemixRes",
   }
 )
 
+
+#' @rdname showall-methods
+#' @exportMethod showall
+
 # Could be print, with only head of data
 setMethod("showall","SaemixRes",
   function(object) {
@@ -511,6 +582,114 @@ setMethod("showall","SaemixRes",
   }
 )
 
+####################################################################################
+####			SaemixRes class - extracting residuals	and fitted values		####
+####################################################################################
+# resid.lm
+#           function (object, type = c("working", "response", "deviance", 
+#                                      "pearson", "partial"), ...) 
+#           {
+#             type <- match.arg(type)
+#             r <- object$residuals
+#             res <- switch(type, working = , response = r, deviance = , 
+#                           pearson = if (is.null(object$weights)) r else r * sqrt(object$weights), 
+#                           partial = r)
+#             res <- naresid(object$na.action, res)
+#             if (type == "partial") 
+#               res <- res + predict(object, type = "terms")
+#             res
+#           }
+
+#' Extract Model Residuals
+#' 
+#' residuals is a generic function which extracts model residuals from objects returned by modelling functions. The  abbreviated form resid is an alias for residuals
+#' 
+#' @name resid.saemix
+#' @aliases residuals.SaemixRes resid.SaemixRes
+#' @aliases residuals.saemix residuals resid resid.SaemixObject residuals.SaemixObject
+#' 
+#' @param object an SaemixRes or an SaemixObject object
+#' @param type string determining which residuals are extracted. Possible values are: "ires" (individual residuals, default), "wres" (weighted population residuals), "npde" (normalised prediction distribution errors), "pd" (prediction discrepancies), "iwres" (individual weighted residuals) and "icwres" (conditional individual weighted residuals). See user guide for details.
+#' @param ... further arguments to be passed to or from other methods
+#' 
+#' @return By default, individual residuals are extracted from the model object
+#' 
+#' @docType methods
+#' @keywords methods
+#' @export resid.SaemixRes
+
+resid.SaemixRes<-function (object, type = c("ires", "wres", "npde", "pd", "iwres", "icwres"), ...) {
+            type <- match.arg(type)
+            res <- switch(type, ires=object@ires, wres=object@wres, npde=object@npde, iwres=object@iwres, icwres=object@icwres, pd=object@pd)
+            res
+}
+
+
+#' Extract Model Predictions
+#' 
+#' fitted is a generic function which extracts model predictions from objects returned by modelling functions
+#' 
+#' @name fitted.saemix
+#' @aliases fitted fitted.SaemixRes fitted.SaemixObject
+#' 
+#' @param object an object of type SaemixRes or SaemixObject
+#' @param type string determining which predictions are extracted. Possible values are: "ipred" (individual predictions obtained using the mode of the individual distribution for each subject, default), "ypred" (population predictions obtained using the population parameters f(E(theta))), "ppred" (mean of the population predictions (E(f(theta)))) and "icpred" (individual predictions obtained using the conditional mean of the individual distribution). See user guide for details.
+#' @param ... further arguments to be passed to or from other methods
+#' 
+#' @return Model predictions
+#' 
+#' @docType methods
+#' @keywords methods
+#' @export fitted.SaemixRes
+
+fitted.SaemixRes<-function (object, type = c("ipred", "ypred", "ppred", "icpred"), ...) {
+  type <- match.arg(type)
+  pred <- switch(type, ipred=object@ipred, ypred=object@ypred, ppred=object@ppred, ipred=object@ipred, icpred=object@icpred)
+  pred
+}
+
+####################################################################################
+####			SaemixRes class - variance-covariance matrix		####
+####################################################################################
+
+#' Extracts the Variance-Covariance Matrix for a Fitted Model Object
+#' 
+#' Returns the variance-covariance matrix of the main parameters of a fitted model object
+#' 
+#' @name vcov
+#' @aliases vcov vcov.SaemixRes vcov.SaemixObject ##vcov,SaemixRes vcov,SaemixObject
+#' 
+#' @param object a fitted object from a call to saemix
+#' @param ...	further arguments to be passed to or from other methods
+#' 
+#' @return A matrix of the estimated covariances between the parameter estimates in model. In saemix, this matrix is obtained as the inverse of the Fisher Information Matrix computed by linearisation
+#' 
+#' @export vcov.SaemixRes
+#' @export vcov.SaemixObject
+
+vcov.SaemixRes<-function(object, ...) {
+  object@fim
+}
+
+vcov.SaemixObject<-function(object, ...) {
+  vcov(object@results)
+}
+
+# setMethod("vcov","SaemixRes",
+#           function (object, ...) 
+#           {
+#             object@fim
+#           }
+# )
+# 
+# setMethod("vcov","SaemixObject",
+#           function (object, ...) 
+#           {
+#             vcov(object@results)
+#           }
+# )
+
+
 ####################################################################################
 ####			SaemixRes class - method to plot			####
 ####################################################################################
